@@ -9,6 +9,7 @@
   const webidl = window.__bootstrap.webidl;
   const { HTTP_TOKEN_CODE_POINT_RE } = window.__bootstrap.infra;
   const { DOMException } = window.__bootstrap.domException;
+  const { defineEventHandler } = window.__bootstrap.event;
   const { Blob } = globalThis.__bootstrap.file;
   const {
     ArrayBuffer,
@@ -16,16 +17,11 @@
     ArrayPrototypeJoin,
     DataView,
     ErrorPrototypeToString,
-    ObjectDefineProperty,
-    Map,
-    MapPrototypeGet,
-    MapPrototypeSet,
     Set,
     Symbol,
     String,
     StringPrototypeToLowerCase,
     StringPrototypeEndsWith,
-    FunctionPrototypeCall,
     RegExpPrototypeTest,
     ObjectDefineProperties,
     ArrayPrototypeMap,
@@ -65,45 +61,6 @@
   const CLOSING = 2;
   const CLOSED = 3;
 
-  const handlerSymbol = Symbol("eventHandlers");
-  function makeWrappedHandler(handler) {
-    function wrappedHandler(...args) {
-      if (typeof wrappedHandler.handler !== "function") {
-        return;
-      }
-      return FunctionPrototypeCall(wrappedHandler.handler, this, ...args);
-    }
-    wrappedHandler.handler = handler;
-    return wrappedHandler;
-  }
-  // TODO(lucacasonato) reuse when we can reuse code between web crates
-  function defineEventHandler(emitter, name) {
-    // HTML specification section 8.1.5.1
-    ObjectDefineProperty(emitter, `on${name}`, {
-      get() {
-        if (!this[handlerSymbol]) {
-          return null;
-        }
-        return MapPrototypeGet(this[handlerSymbol], name)?.handler;
-      },
-      set(value) {
-        if (!this[handlerSymbol]) {
-          this[handlerSymbol] = new Map();
-        }
-        let handlerWrapper = MapPrototypeGet(this[handlerSymbol], name);
-        if (handlerWrapper) {
-          handlerWrapper.handler = value;
-        } else {
-          handlerWrapper = makeWrappedHandler(value);
-          this.addEventListener(name, handlerWrapper);
-        }
-        MapPrototypeSet(this[handlerSymbol], name, handlerWrapper);
-      },
-      configurable: true,
-      enumerable: true,
-    });
-  }
-
   const _readyState = Symbol("[[readyState]]");
   const _url = Symbol("[[url]]");
   const _rid = Symbol("[[rid]]");
@@ -324,10 +281,10 @@
       const sendTypedArray = (ta) => {
         this[_bufferedAmount] += ta.byteLength;
         PromisePrototypeThen(
-          core.opAsync("op_ws_send", {
-            rid: this[_rid],
+          core.opAsync("op_ws_send", this[_rid], {
             kind: "binary",
-          }, ta),
+            value: ta,
+          }),
           () => {
             this[_bufferedAmount] -= ta.byteLength;
           },
@@ -348,10 +305,9 @@
         const d = core.encode(string);
         this[_bufferedAmount] += d.byteLength;
         PromisePrototypeThen(
-          core.opAsync("op_ws_send", {
-            rid: this[_rid],
+          core.opAsync("op_ws_send", this[_rid], {
             kind: "text",
-            text: string,
+            value: string,
           }),
           () => {
             this[_bufferedAmount] -= d.byteLength;
@@ -456,8 +412,7 @@
             break;
           }
           case "ping": {
-            core.opAsync("op_ws_send", {
-              rid: this[_rid],
+            core.opAsync("op_ws_send", this[_rid], {
               kind: "pong",
             });
             break;
